/*
 * BLTouch.c
 *
 * Created: 25.08.2019 21:25:31
 * Author : Tobias N.
 */
 
#define F_CPU 31250UL

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/power.h>

volatile uint8_t triggerFlag = 0;

void blTouch_mode(uint8_t mode);

ISR(INT0_vect) {
	
	cli();
	triggerFlag = 1;	// BLTouch triggered
	EIFR |= _BV(INTF0); // clear interrupt flag
	sei();
	
}

int main(void) {
	
	clock_prescale_set(clock_div_256); // Divide main clock by 256
	
	ACSR |= _BV(ACD);		// AC not needed, hence disable to reduce power consumption
	PRR  |= _BV(PRADC);		// Shut down ADC to reduce power consumption
	
	DDRB |= _BV(PORTB0);
	DDRB &= ~_BV(PORTB1);
	PUEB |= _BV(PORTB1);
	
	EICRA |= _BV(ISC01);	// Falling edge interrupt enable
	EIMSK |= _BV(INT0);		// Enable external interrupt
	
    TCCR0A |= _BV(WGM01);
    TCCR0B |= _BV(WGM03) | _BV(WGM02) | _BV(CS00); // PWM mode: Fast PWM using ICR0 as TOP // clk select: No prescaling
	
	/*
	Calculate TOP value for PWMfreq 50Hz
	
	PWMfreq = fclk_i/o / (N * (1 + TOP))
	TOP = (fclk_i/o / PWMfreq) / N - 1
	TOP = (31250 / 50) / 1 - 1
	TOP = 624
	*/
	
	cli();
	ICR0 = 624; // Set TOP value
	sei();
	
    while (1) {
		
		static uint8_t probingFlag = 0;
		
		while((PINB & _BV(PORTB1)) == 0) {
			
			TCCR0A |= _BV(COM0A1);	// Enable non-inverting compare output mode operation
			
			blTouch_mode(10);		// Button released within 0.5 sec -> BLTouch mode: 10
			_delay_ms(500);
			
			if(PINB & _BV(PORTB1)) {
				probingFlag = 1;	// Set flag to indicate on-going probing operation
				break;				
			}
			
			blTouch_mode(90);		// Button released after 0.5 sec -> BLTouch mode: 90	
			_delay_ms(500);
			
			if(PINB & _BV(PORTB1)) {
				probingFlag = 0;	// Clear flag to indicate end of probing operation
				break;
			}
			
			blTouch_mode(160);		// Button released after 1 sec -> BLTouch mode: 160
			_delay_ms(500);
			
			if(PINB & _BV(PORTB1)) {
				probingFlag = 0;	// Clear flag to indicate end of probing operation
				break;
			}
				
		}
		
		if(probingFlag == 1 && triggerFlag == 1) {
			
			blTouch_mode(90);
			_delay_ms(1000);		// keep push-pin up for one more second to allow the machine to get out of probing range
			blTouch_mode(10);
			triggerFlag = 0;
			
		}
    }
}

void blTouch_mode(uint8_t mode) {
	
	/*
	BLTouch modes:
	Push-pin down:	647s	(10)
	Push-pin up:	1473s	(90)
	Alarm release:	2194s	(160)
	*/
	
	switch(mode) {
		case 10: {
			cli();
			OCR0A = 19; // approximately 650s
			sei();
		}
		break;
		case 90: {
			cli();
			OCR0A = 44; // approximately 1470s
			sei();
		}
		break;
		case 160: {
			cli();
			OCR0A = 66; // approximately 2200s
			sei();
		}
		break;
	}
}